package soma;

import java.awt.FileDialog;
import java.awt.Frame;
import java.io.File;
import java.io.IOException;

import javax.swing.JOptionPane;

public class Defi {

	private static enum Mode { CONSOLE, GUI; }
	private static Mode mode = Mode.CONSOLE;
	
	public static void message(String s) {
		if (mode==Mode.CONSOLE) System.out.println(s);
		if (mode==Mode.GUI) JOptionPane.showMessageDialog(null, s, "Information", JOptionPane.INFORMATION_MESSAGE);;
	}

	public static void main(String[] args) {

		String pathname = null;
		Display display = null;

		if (args.length==0)	Defi.mode = Mode.GUI;
		
		switch(Defi.mode)	{
		case GUI:
			// initialise l'affichage 3D
			try {
				System.out.print("Initialize 3D display... ");
				display = new Display();
				System.out.println("ok.");
			} catch(Throwable thr) {
				System.out.println("3D Display disabled : "+thr.getClass().getName()+" -> "+thr.getMessage());
			}
			
			// selection du fichier
			FileDialog filedialog = new FileDialog((Frame)null, "Choose an image file");
			filedialog.setVisible(true);
			String filename = filedialog.getFile();
			String directory = filedialog.getDirectory();
			if (filename == null) System.exit(1);
			pathname = directory + File.separator + filename;
			break;
		case CONSOLE:
			pathname = args[0];
			break;
		}
		
		// chargement du fichier
		System.out.print("loading file " +pathname+" ... ");
		int[][] data = null;
		try {
			data = FileLoader.loadPuzzle(pathname);
		} catch (IOException e) {
			message("ERROR : " + e.getMessage());
			System.exit(1);
		}
		System.out.println("ok.");
		
		// construction du puzzle
		Puzzle puzzle = new Puzzle(data);

		// resolution du puzzle
		System.out.print("Solving... ");
		Solver solver = new Solver();
		long t0=System.currentTimeMillis();
		boolean success = solver.solve(puzzle);
		long t1=System.currentTimeMillis();
		long duration = t1-t0;
		System.out.println("done.");
		
		String result;
		if (success)
			result = "Solution trouve ("+duration+"ms)";
		else
			result = "Pas de solution ("+duration+"ms)";
		
		// affichage du rsultat
		if (display!=null) {
			display.setTitle(result);
			display.show(puzzle,solver.getSolution());
		} else {
			message(result);
			if (success) message(solver.getSolutionTxt());
			System.exit(0);
		}
	}
}
