package soma;

import java.awt.Color;
import java.util.Map;
import java.util.Map.Entry;

import javax.media.j3d.AmbientLight;
import javax.media.j3d.Appearance;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Canvas3D;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.Light;
import javax.media.j3d.Material;
import javax.media.j3d.Node;
import javax.media.j3d.PointLight;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.swing.JFrame;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;

/**
 * Classe affichant le puzzle en utilisant Java3D
 * 
 * @author xphilippeau
 *
 */
public class Display {
	
	// Swing Display
	private JFrame rootframe = null;

	// Java3D
	private SimpleUniverse simpleU;
	private BranchGroup scene;
    private Canvas3D canvas3D;

    public Display() {
    	// create Java3D stuff
    	this.canvas3D = new Canvas3D(SimpleUniverse.getPreferredConfiguration());
		this.simpleU = new SimpleUniverse(this.canvas3D);
		this.scene = new BranchGroup();
		// create Swing stuff
		this.rootframe = new JFrame("DVP - Defi 5");
		this.rootframe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}
    
    public void setTitle(String title) {
    	this.rootframe.setTitle(title);
    }
    
    public void show(Puzzle puzzle, Map<SomaElement,Shape> solution) {

    	// create puzzle object
		TransformGroup puzzle3d = new TransformGroup();
		if (solution.size()==0)
    		puzzle3d.addChild( new Shape3D(puzzle.cubes, 0.9f, Color.LIGHT_GRAY).node );
    	else
    		for(Entry<SomaElement,Shape> entry:solution.entrySet()) {
    			SomaElement soma = entry.getKey();
    			Shape shape = entry.getValue();
    			puzzle3d.addChild( new Shape3D(shape.cubes, 1.0f, soma.color).node );
    		}
    	
    	// center to origin
    	double cx = (puzzle.minX+puzzle.maxX)/2;
    	double cy = (puzzle.minY+puzzle.maxY)/2;
    	double cz = (puzzle.minZ+puzzle.maxZ)/2;
    	Transform3D translate = new Transform3D();
		translate.set(new Vector3d(-cx,-cy,-cz));
		puzzle3d.setTransform(translate);
	    
		// add object to scene and compute bounding sphere
		this.scene.setBoundsAutoCompute(true);
		this.scene.addChild(puzzle3d);
		BoundingSphere bounds = (BoundingSphere)this.scene.getBounds();

    	// add directional light
		Light dlight = new PointLight(new Color3f (0.75f, 0.75f, 0.75f), new Point3f (20f,23f,27f), new Point3f (1,0,0) );
		dlight.setInfluencingBounds (bounds);
		this.scene.addChild(dlight);

		Light dlight2 = new DirectionalLight ( new Color3f (0.5f, 0.5f, 0.5f), new Vector3f (0.5f, 1.0f, 1.5f) );
		dlight2.setInfluencingBounds (bounds);
		this.scene.addChild(dlight2);

		// add ambient light
		Light ambientlight = new AmbientLight(new Color3f(0.3f, 0.3f, 0.3f));
		ambientlight.setInfluencingBounds (bounds);
		this.scene.addChild(ambientlight);

		// compile the scene and add to the universe
		this.scene.compile();
		this.simpleU.addBranchGraph(scene);

    	// Viewpoint settings
		ViewingPlatform vp = simpleU.getViewingPlatform();
		Point3d eye = new Point3d(12,12,12);
		Point3d target = new Point3d(0,0,0);
		Transform3D camera = new Transform3D();
		camera.lookAt(eye, target, new Vector3d(0,0,1));
		camera.setScale(9.0/bounds.getRadius());
		camera.invert();
		vp.getViewPlatformTransform().setTransform(camera);
		
		// add Orbit Behavior (mouse control)
    	OrbitBehavior orbit = new OrbitBehavior(canvas3D, OrbitBehavior.REVERSE_ALL);
		orbit.setSchedulingBounds(bounds);
		orbit.setTransFactors(1.0,1.0);
		orbit.setRotFactors(0.5,0.5);
		orbit.setZoomFactor(0.2);
		orbit.setProportionalZoom(true);
		orbit.setRotationCenter(new Point3d(0,0,0));
		vp.setViewPlatformBehavior(orbit);
		
		// Swing rendering
		this.rootframe.setSize(512, 512);
		this.rootframe.add(this.canvas3D);
		this.rootframe.setResizable(true);
		this.rootframe.setVisible(true);
    }
	
    private class Shape3D {
    	
    	Node node;
    	
    	public Shape3D(int[] cubes, float size, Color color) {
    		float BSIZE=size/2;
        	
    		Material material = new Material();
    		material.setLightingEnable(true);
    		material.setAmbientColor( new Color3f(color) );
    		material.setDiffuseColor(  new Color3f(color) );
    		material.setSpecularColor( new Color3f(0.25f,0.25f,0.25f) );
    		material.setShininess(10.0f);
 
    		Appearance appearance = new Appearance();
    		appearance.setMaterial(material);
    		
    		BranchGroup shape = new BranchGroup();
    		for(int cube : cubes) {
    			int[] coord = Coordinates.ToCoord(cube);
    			
    			Box box = new Box(BSIZE,BSIZE,BSIZE,Box.GENERATE_NORMALS,appearance);

    			TransformGroup transform = new TransformGroup();
    			Transform3D translate = new Transform3D();
    			translate.set(new Vector3d(coord[0],coord[1],coord[2]));
    			transform.setTransform(translate);
    			transform.addChild(box);
    			
    			shape.addChild(transform);
    		}
    		this.node = shape;
		}
    }

}
