package soma;

import java.io.File;
import java.io.IOException;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Classe utilitaire pour la lecture du fichier CSV
 * 
 * @author xphilippeau
 *
 */
public class FileLoader {

	/**
	 * Lecture du fichier CSV contenant les points du puzzle
	 * 
	 * @param pathname chemin du fichier
	 * @return les coordonnes des 27 points dans un tableau int[27][3]
	 * @throws IOException en cas d'erreur ou d'invalidit 
	 */
	public static int[][] loadPuzzle(String pathname) throws IOException {
		if (pathname == null) throw new IOException("invalid file name");
		
		// Accs au fichier
		File f = new File(pathname);
		if (!f.exists()) throw new IOException("file not found");
		if (!f.isFile()) throw new IOException("is not a file");
		if (!f.canRead()) throw new IOException("file not readable");

		// structure retourne
		int[][] data = new int[27][3];
		int count=0;

		// format d'une ligne
		Pattern pattern = Pattern.compile("^([0-9]),([0-9]),([0-9])$");
		
		// analyse ligne  ligne
		Scanner filescanner = new Scanner(f);
		try {
			while (filescanner.hasNextLine()) {
				if (count>=data.length) throw new IOException("Too many lines");
				String line = filescanner.nextLine();
				Matcher matcher = pattern.matcher(line);
				if (!matcher.matches()) throw new IOException("invalid line (#"+(count+1)+")");
				data[count][0]=Integer.parseInt(matcher.group(1));
				data[count][1]=Integer.parseInt(matcher.group(2));
				data[count][2]=Integer.parseInt(matcher.group(3));
				count++;
			}
		} finally {
			filescanner.close();
		}
		
		if (count!=data.length)
			throw new IOException("invalid file format");
		
		return data;
	}
	
}
