package soma;

import java.util.Arrays;
import java.util.List;

/**
 * Classe reprsentant une forme = un ensemble de cube
 * 
 * @author xphilippeau
 *
 */
public class Shape {
	private static final int X=0,Y=1,Z=2;
	
	// Bounding box
	public final int minX,minY,minZ,maxX,maxY,maxZ;
	
	// liste des cubes
	public final int[] cubes;
	
	/**
	 * @param data int[N][3] liste des coordonnes des N cubes composant le puzzle
	 */
	public Shape(int[][] data) {
		// calcule la bounding box
		int[] min = {data[0][X],data[0][Y],data[0][Z]};
		int[] max = {data[0][X],data[0][Y],data[0][Z]};
		for(int[] c:data) {
			min[X] = Math.min(min[X], c[X]);
			min[Y] = Math.min(min[Y], c[Y]);
			min[Z] = Math.min(min[Z], c[Z]);
			max[X] = Math.max(max[X], c[X]);
			max[Y] = Math.max(max[Y], c[Y]);
			max[Z] = Math.max(max[Z], c[Z]);
		}
		minX=min[X]; minY=min[Y]; minZ=min[Z];
		maxX=max[X]; maxY=max[Y]; maxZ=max[Z];
		
		// cration de la liste de cube. 
		// Les coordonnes 3D du cube sont convertie en un entier 
		this.cubes = new int[data.length];
		for(int i=0;i<data.length;i++) 
			this.cubes[i]=Coordinates.ToIndex(data[i][X],data[i][Y],data[i][Z]);
		
		// trie la lise d'entier (pour faciliter les comparaisons)
		Arrays.sort(this.cubes);
	}	
	
	/**
	 * Compare deux formes
	 * 
	 * @param s forme a comparer
	 * @return true si les 2 formes sont identiques
	 */
	public boolean equals(Shape s) {
		if (this.cubes.length!=s.cubes.length) return false;
		// comme les 2 tableaux sont tris, on compare element  element
		for(int i=0;i<this.cubes.length;i++) 
			if (this.cubes[i]!=s.cubes[i]) return false;
		return true;
	}
	
	/**
$	 * @return true si la forme 's' fait partie de la la list
	 */
	public static boolean contains(List<Shape> list, Shape s) {
		for(Shape s0:list)
			if(s.equals(s0)) return true;
		return false;
	}
}
