package soma;

/**
 * Classe effectuant des transormations 3D sur une forme
 * 
 * @author xphilippeau
 *
 */
public class Transform {

	private static final int X=0,Y=1,Z=2;
	
	/**
	 * Cre une forme par translation+rotation
	 * 
	 * @param data coordones initiales
	 * @param tx,ty,tz la translation a effectuer
	 * @param rot la rotation a effectuer
	 * @return une nouvelle forme
	 */
	public static Shape transform(int[][] data,int tx, int ty, int tz, RotationMatrix rot) {
		int[][] cdata = new int[data.length][3];
		// rotation de l'lement
		for(int i=0;i<data.length;i++) {
			int[] coord = data[i];
			cdata[i][X] = coord[X]*rot.matrix[0][0] + coord[Y]*rot.matrix[1][0] + coord[Z]*rot.matrix[2][0];
			cdata[i][Y] = coord[X]*rot.matrix[0][1] + coord[Y]*rot.matrix[1][1] + coord[Z]*rot.matrix[2][1];
			cdata[i][Z] = coord[X]*rot.matrix[0][2] + coord[Y]*rot.matrix[1][2] + coord[Z]*rot.matrix[2][2];
		}
		// recherche des coordonns minimales
		int[] min = {cdata[0][X],cdata[0][Y],cdata[0][Z]};
		for(int[] c:cdata) {
			min[X] = Math.min(min[X], c[X]);
			min[Y] = Math.min(min[Y], c[Y]);
			min[Z] = Math.min(min[Z], c[Z]);
		}
		// alignement des coordonns minimales en (0,0,0) + translation de l'lement
		for(int[] c:cdata) {
			c[X]-=min[X]; c[X]+=tx; 
			c[Y]-=min[Y]; c[Y]+=ty;
			c[Z]-=min[Z]; c[Z]+=tz;
			if (c[X]<0 || c[X]>9) return null; // out of bound
			if (c[Y]<0 || c[Y]>9) return null; // out of bound
			if (c[Z]<0 || c[Z]>9) return null; // out of bound
		}
		return new Shape(cdata);
	}
}
